/*
 * add support for switching the splash screen on boot
 */
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include "splash.h"

static int verbose = 0;

/* nop implementation
 */
static void nop(void)
{
}

static struct splash_ops nop_ops = {
	.splash_on = nop,
	.splash_off = nop
};

/*
 *  bootsplash implementation
 */
#define BOOTSPLASH_CTL	"/proc/splash"

static int bootsplash_exists(void)
{
	struct stat sb;

	if (stat(BOOTSPLASH_CTL, &sb) == -1)
		return 0;

	if (S_ISREG(sb.st_mode))
		return 1;

	return 0;
}

/* write msg to splash control */
static void bootsplash_msg(const char *msg, size_t size)
{
	int fd;
	size_t written;

	fd = open(BOOTSPLASH_CTL, O_WRONLY);
	if (fd == -1) {
		if (verbose)
			printf("cannot open %s\n", BOOTSPLASH_CTL);
		return;
	}

	written = write(fd, msg, size);
	if (written != size) {
		if (verbose)
			printf("size = %zd, written = %zd\n", size, written);
	}

	close(fd);
}

static void bootsplash_on(void)
{
	if (verbose)
		printf("setting bootsplash silent\n");
	bootsplash_msg("silent\n", 7);
}

static void bootsplash_off(void)
{
	if (verbose)
		printf("setting bootsplash verbose\n");
	bootsplash_msg("verbose\n", 8);
}

static struct splash_ops bootsplash_ops = {
	.splash_on = bootsplash_on,
	.splash_off = bootsplash_off
};

/*
 * Initialisation
 */
void splash_init(struct splash_ops **ops)
{
	if (bootsplash_exists())
		*ops = &bootsplash_ops;
	else
		*ops = &nop_ops;
}

void splash_set_verbose(void)
{
	verbose = 1;
}

